using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.OpenApi.Models;

namespace RESTbookStatusCodes
{
    public class Startup
    {
        private const string AllowGetPut = "allowGetPut";
        private const string AllowAnythingFromZealand = "allowAnythingFromZealand";
        private const string AllowAnything = "allowAnything";

        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddCors(options =>
            {
                options.AddPolicy(AllowAnythingFromZealand,
                    builder =>
                        builder.WithOrigins("http://zealand.dk")
                            .AllowAnyHeader()
                            .AllowAnyMethod());
                options.AddPolicy(AllowGetPut,
                    builder =>
                        builder.AllowAnyOrigin()
                            .WithMethods("GET", "PUT")
                            .AllowAnyHeader());
                options.AddPolicy(AllowAnything, // similar to * in Azure
                    builder =>
                        builder.AllowAnyOrigin()
                            .AllowAnyMethod()
                            .AllowAnyHeader());
            });

            services.AddControllers();
            services.AddSwaggerGen(c =>
            {
                c.SwaggerDoc("v1", new OpenApiInfo { Title = "Items API", Version = "v1.0" });
            });
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseRouting();

            //app.UseCors(AllowAnythingFromZealand);
            app.UseCors(AllowAnything);

            app.UseAuthorization();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });

            app.UseSwagger();

            app.UseSwaggerUI(c =>
                c.SwaggerEndpoint("/swagger/v1/swagger.json", "Items API v1.0")
            );
        }
    }
}
